<?php

namespace Modules\Sd\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Sd\Entities\CollectingFinalDocument;
use Modules\Sd\Entities\CollectingFinalDocumentAttachment;
use Modules\Sd\Entities\Shipment;
use Modules\Sd\Entities\ShippingAgent;

class CollectingFinalDocumentController extends Controller
{

    public function getAllCFDData()
    {
        try {
            $query = 'SELECT * FROM sd_collecting_final_document ORDER BY `created_at` DESC';
            $cfd_data = DB::select($query);
            $result = array();

            foreach ($cfd_data as $data) {
                array_push($result, [
                    "customer_order_id" => $data->customer_order_id,
                    "factory_po" => $data->factory_po,
                    "shipment_id" => $data->shipment_id,
                    "cfd_id" => $data->cfd_id,
                ]);
            }
            $responseBody = $this->responseBody(true, "CollectingFinalDocumentController", "getAllCFDData", $result);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "CollectingFinalDocumentController", "getAllCFDData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function saveData(Request $request)
    {
        try {
            $cfd_data = new CollectingFinalDocument();
            $cfd_data->customer_order_id = $request->get('customer_order_id');
            $cfd_data->factory_po = $request->get('factory_po');
            $cfd_data->shipment_id = $request->get('shipment_id');
            $cfd_data->note = $request->get('note');

            if ($cfd_data->save()) {
                // $this->sendEmail();
                $attachments = CollectingFinalDocumentAttachment::where([['cfd_attachment_id', '=', 0], ['token', '=', $request->token]])->get();
                foreach ($attachments as $attachment) {
                    $attachment->cfd_id = $cfd_data->cfd_id;
                    $attachment->update();
                }
                $attachments = CollectingFinalDocumentAttachment::where('token', '=', $request->token)->get();
                foreach ($attachments as $attachment) {
                    $attachment->cfd_id = $cfd_data->cfd_id;
                    $attachment->update();
                }
            }

            $responseBody = $this->responseBody(true, "CollectingFinalDocumentController", "saveData", $request->token);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "CollectingFinalDocumentController", "saveData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function update(Request $request, $id)
    {
        try {
            $cfd_data = CollectingFinalDocument::find($id);
            $cfd_data->customer_order_id = $request->get('customer_order_id');
            $cfd_data->factory_po = $request->get('factory_po');
            $cfd_data->shipment_id = $request->get('shipment_id');
            $cfd_data->note = $request->get('note');
            if ($cfd_data->update()) {
                $attachments = CollectingFinalDocumentAttachment::where([['cfd_attachment_id', '=', 0], ['token', '=', $request->token]])->get();
                foreach ($attachments as $attachment) {
                    $attachment->cfd_id = $cfd_data->cfd_id;
                    $attachment->update();
                }
                $attachments = CollectingFinalDocumentAttachment::where('token', '=', $request->token)->get();
                foreach ($attachments as $attachment) {
                    $attachment->cfd_id = $cfd_data->cfd_id;
                    $attachment->update();
                }
            }

            $responseBody = $this->responseBody(true, "CollectingFinalDocumentController", "update", $request->token);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "CollectingFinalDocumentController", "update", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function delete(Request $request, $id)
    {
        try {
            $cfd_data = CollectingFinalDocument::where('cfd_id', '=', $id);
            $rec_data_attachments = CollectingFinalDocumentAttachment::where('cfd_attachment_id', '=', $id)->get();

            if ($cfd_data->delete()) {
                foreach ($rec_data_attachments as $rec_data_attachment) {
                    $rec_data_attachment->delete();
                }
            }

            $responseBody = $this->responseBody(true, "CollectingFinalDocumentController", "delete", "success");
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "CollectingFinalDocumentController", "delete", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function getCFDData($id)
    {
        try {
            $recordHead = CollectingFinalDocument::find($id);

            $responseBody = $this->responseBody(true, "CollectingFinalDocumentController", "getCFDData", $recordHead);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "CollectingFinalDocumentController", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }
    public function getShipmentData()
    {
        try {
            $shipmet = Shipment::all();

            $responseBody = $this->responseBody(true, "CollectingFinalDocumentController", "getShipmentData", $shipmet);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "CollectingFinalDocumentController", "getShipmentData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }
    public function getShippingAgentData()
    {
        try {
            $shipping_agent = ShippingAgent::all();

            $responseBody = $this->responseBody(true, "CollectingFinalDocumentController", "getShippingAgentData", $shipping_agent);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "CollectingFinalDocumentController", "getShippingAgentData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function getCustomerOrderData($id)
    {
        try {
            $query = 'SELECT
            sd_shipment.customer_order_id,
            sd_shipment.shipment_id,
            pp_customer_orders.factory_po_num
        FROM
            sd_shipment
            LEFT JOIN pp_customer_orders ON sd_shipment.customer_order_id = pp_customer_orders.order_id
        WHERE
            sd_shipment.shipment_id = ' . $id;
            $orders = DB::select($query);
            $result = array();

            foreach ($orders as $order) {
                array_push($result, [
                    "customer_order_id" => $order->customer_order_id,
                    "factory_po_num" => $order->factory_po_num,
                    "shipment_id" => $order->shipment_id
                ]);
            }


            $responseBody = $this->responseBody(true, "CollectingFinalDocumentController", "getCustomerOrderData", $result);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "CollectingFinalDocumentController", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function uploadScheduleAttachment(Request $request)
    {

        $description = $request->get('foo');
        $token = $request->get('attachmnet_token');
        $file = $request->file('file');
        $file_name = $file->getClientOriginalName();
        $filename = 'http://127.0.0.1:8000/collecting_final_document/' . uniqid() . '_' . time() . '.' . str_replace(' ', '_', $file_name);
        //$filename = 'https://erp.rbs.lk/order/' . uniqid() . '_' . time() . '.' .str_replace(' ', '_', $file_name);
        $file->move(public_path('collecting_final_document'), $filename);

        $cfd_data = new CollectingFinalDocumentAttachment();
        $cfd_data->cfd_id = 0;
        $cfd_data->description = $description;
        $cfd_data->file_path = $filename;
        $cfd_data->token = $token;
        return $cfd_data->save();
    }

    public function allScheduleAttachment($id)
    {
        try {

            $attachment = DB::select('SELECT * FROM sd_collecting_final_document_attachments WHERE  cfd_id = "' . $id . '" OR cfd_id = "0"');
            $responseBody = $this->responseBody(true, "CollectingFinalDocumentController", "allScheduleAttachment", $attachment);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "CollectingFinalDocumentController", "allScheduleAttachment", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function deleteScheduleAttachment($id)
    {
        try {
            $rec_data = CollectingFinalDocumentAttachment::where('cfd_attachment_id', '=', $id)->get();
            foreach ($rec_data as $data) {
                $schedule = $data;
                $data->delete();
            }
            $schedule = true;
            $responseBody = $this->responseBody(true, "CollectingFinalDocumentController", "delete schedule attachment", $schedule);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "CollectingFinalDocumentController", "delete schedule attachment", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
