<?php

namespace Modules\Sd\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Sd\Entities\Shipment;
use Modules\Sd\Entities\ShippingAgent;
use Modules\Sd\Entities\SpecInsForLoading;
use Modules\Sd\Entities\SpecInsForLoadingAttachment;

class SpecInsForLoadingController extends Controller
{

    public function getAllSpInsLoadData()
    {
        try {
            $query = 'SELECT * FROM sd_spec_ins_for_loading ORDER BY `created_at` DESC';
            $cfd_data = DB::select($query);
            $result = array();

            foreach ($cfd_data as $data) {
                array_push($result, [
                    "customer_order_id" => $data->customer_order_id,
                    "factory_po" => $data->factory_po,
                    "shipment_id" => $data->shipment_id,
                    "sp_ins_loading_id" => $data->sp_ins_loading_id,
                ]);
            }
            $responseBody = $this->responseBody(true, "SpecInsForLoadingController", "getAllSpInsLoadData", $result);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SpecInsForLoadingController", "getAllSpInsLoadData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function saveData(Request $request)
    {
        try {
            $sp_ins_data = new SpecInsForLoading();
            $sp_ins_data->customer_order_id = $request->get('customer_order_id');
            $sp_ins_data->factory_po = $request->get('factory_po');
            $sp_ins_data->shipment_id = $request->get('shipment_id');
            $sp_ins_data->note = $request->get('note');

            if ($sp_ins_data->save()) {
                // $this->sendEmail();
                $attachments = SpecInsForLoadingAttachment::where([['sp_ins_loading_attachment_id', '=', 0], ['token', '=', $request->token]])->get();
                foreach ($attachments as $attachment) {
                    $attachment->sp_ins_loading_id = $sp_ins_data->sp_ins_loading_id;
                    $attachment->update();
                }
                $attachments = SpecInsForLoadingAttachment::where('token', '=', $request->token)->get();
                foreach ($attachments as $attachment) {
                    $attachment->sp_ins_loading_id = $sp_ins_data->sp_ins_loading_id;
                    $attachment->update();
                }
            }

            $responseBody = $this->responseBody(true, "SpecInsForLoadingController", "saveData", $request->token);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SpecInsForLoadingController", "saveData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function update(Request $request, $id)
    {
        try {
            $sp_ins_data = SpecInsForLoading::find($id);
            $sp_ins_data->customer_order_id = $request->get('customer_order_id');
            $sp_ins_data->factory_po = $request->get('factory_po');
            $sp_ins_data->shipment_id = $request->get('shipment_id');
            $sp_ins_data->note = $request->get('note');
            if ($sp_ins_data->update()) {
                $attachments = SpecInsForLoadingAttachment::where([['sp_ins_loading_attachment_id', '=', 0], ['token', '=', $request->token]])->get();
                foreach ($attachments as $attachment) {
                    $attachment->sp_ins_loading_id = $sp_ins_data->sp_ins_loading_id;
                    $attachment->update();
                }
                $attachments = SpecInsForLoadingAttachment::where('token', '=', $request->token)->get();
                foreach ($attachments as $attachment) {
                    $attachment->sp_ins_loading_id = $sp_ins_data->sp_ins_loading_id;
                    $attachment->update();
                }
            }

            $responseBody = $this->responseBody(true, "SpecInsForLoadingController", "update", $request->token);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SpecInsForLoadingController", "update", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function delete(Request $request, $id)
    {
        try {
            $sp_ins_load_data = SpecInsForLoading::where('sp_ins_loading_id', '=', $id);
            $rec_data_attachments = SpecInsForLoadingAttachment::where('sp_ins_loading_attachment_id', '=', $id)->get();

            if ($sp_ins_load_data->delete()) {
                foreach ($rec_data_attachments as $rec_data_attachment) {
                    $rec_data_attachment->delete();
                }
            }

            $responseBody = $this->responseBody(true, "SpecInsForLoadingController", "delete", "success");
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SpecInsForLoadingController", "delete", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function getSpInsLoadingData($id)
    {
        try {
            $recordHead = SpecInsForLoading::find($id);

            $responseBody = $this->responseBody(true, "SpecInsForLoadingController", "getSpInsLoadingData", $recordHead);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SpecInsForLoadingController", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }
    public function getShipmentData()
    {
        try {
            $shipmet = Shipment::all();

            $responseBody = $this->responseBody(true, "SpecInsForLoadingController", "getShipmentData", $shipmet);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SpecInsForLoadingController", "getShipmentData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }
    public function getShippingAgentData()
    {
        try {
            $shipping_agent = ShippingAgent::all();

            $responseBody = $this->responseBody(true, "SpecInsForLoadingController", "getShippingAgentData", $shipping_agent);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SpecInsForLoadingController", "getShippingAgentData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function getCustomerOrderData($id)
    {
        try {
            $query = 'SELECT
            sd_shipment.customer_order_id,
	        sd_shipment.shipment_id,
            pp_customer_orders.factory_po_num
        FROM
            sd_shipment
            LEFT JOIN pp_customer_orders ON sd_shipment.customer_order_id = pp_customer_orders.order_id
        WHERE
            sd_shipment.shipment_id = ' . $id;
            $orders = DB::select($query);
            $result = array();

            foreach ($orders as $order) {
                array_push($result, [
                    "customer_order_id" => $order->customer_order_id,
                    "factory_po_num" => $order->factory_po_num,
                    "shipment_id" => $order->shipment_id,
                ]);
            }


            $responseBody = $this->responseBody(true, "SpecInsForLoadingController", "getCustomerOrderData", $result);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SpecInsForLoadingController", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function uploadScheduleAttachment(Request $request)
    {

        $description = $request->get('foo');
        $token = $request->get('attachmnet_token');
        $file = $request->file('file');
        $file_name = $file->getClientOriginalName();
        $filename = 'http://127.0.0.1:8000/spec_ins_loading/' . uniqid() . '_' . time() . '.' . str_replace(' ', '_', $file_name);
        //$filename = 'https://erp.rbs.lk/order/' . uniqid() . '_' . time() . '.' .str_replace(' ', '_', $file_name);
        $file->move(public_path('spec_ins_loading'), $filename);

        $sp_ins_loading = new SpecInsForLoadingAttachment();
        $sp_ins_loading->sp_ins_loading_id = 0;
        $sp_ins_loading->description = $description;
        $sp_ins_loading->file_path = $filename;
        $sp_ins_loading->token = $token;
        return $sp_ins_loading->save();
    }

    public function allScheduleAttachment($id)
    {
        try {

            $attachment = DB::select('SELECT * FROM sp_spec_ins_for_loading_attachments WHERE  sp_ins_loading_id = "' . $id . '" OR sp_ins_loading_id = "0"');
            $responseBody = $this->responseBody(true, "SpecInsForLoadingController", "allScheduleAttachment", $attachment);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SpecInsForLoadingController", "allScheduleAttachment", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function deleteScheduleAttachment($id)
    {
        try {
            $rec_data = SpecInsForLoadingAttachment::where('sp_ins_loading_attachment_id', '=', $id)->get();
            foreach ($rec_data as $data) {
                $schedule = $data;
                $data->delete();
            }
            $schedule = true;
            $responseBody = $this->responseBody(true, "SpecInsForLoadingController", "delete schedule attachment", $schedule);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SpecInsForLoadingController", "delete schedule attachment", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
